'' =================================================================================================
''
''   File....... jm_grayenc_demo.spin
''   Purpose.... Greycode encoder test program
''   Author..... Jon "JonnyMac" McPhalen (aka Jon Williams)
''               Copyright (c) 2010 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 17 MAR 2010
''
'' =================================================================================================


con

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                          ' 80 MHz (standard)

  MS_001   = 80_000                                             ' ticks in 1ms


con

  #0, LSBFIRST, MSBFIRST


con

  #1, HOME, #8, BKSP, TAB, LF, CLREOL, CLRDN, CR, #16, CLS      ' PST formmatting control


obj
                                                                
  term : "jm_txserial"                                          ' for terminal output
  enc1 : "jm_grayenc2btn"                                       ' 2-bit, graycode encoder
  enc2 : "jm_grayenc2btn" 


var


pub main | tmp

  term.init(30, 230_400)                                        ' start terminal for test

  enc1.init(3, 5, true,    0, 100, 50)                          ' start encoders  
  enc2.init(6, 8, true, -100, 100,  0)

  pause(5)
  term.tx(CLS)
  term.str(string("P/P Digital Inputs"))
  
  repeat
    term.str(string(HOME, LF, LF))
    
    term.str(string("Encoder1... "))                            ' display encoder 1 
    tmp := enc1.read                                            ' get the value
    if (tmp < 0)                                                ' print sign
      term.tx("-")
    elseif (tmp == 0)
      term.tx(" ")
    else
      term.tx("+")
    term.rjdec(tmp, 3)                                          ' right-justify output
    if enc1.getbutton                                           ' read and show button state
      term.str(string("  Yes", CR))
    else
      term.str(string("  No ", CR))

    term.str(string("Encoder2... "))
    tmp := enc2.read
    if (tmp < 0)
      term.tx("-")
    elseif (tmp == 0)
      term.tx(" ")
    else
      term.tx("+")
    term.rjdec(tmp, 3)
    
    if enc2.getbutton
      term.str(string("  Yes", CR))
    else
      term.str(string("  No ", CR))
    term.tx(CR)
    term.tx(CR)

    term.str(string("74x165..... "))
'   tmp := in165(1, 0, 2)                                       ' default, 8 bits, MSBFIRST
    tmp := in165x(1, 0, 2, 8, LSBFIRST)                         ' alternate, 8 bits, LSBFIRST
    term.bin(tmp, 8)

    pause(50)                                                   ' prevent tx buffer over-run


pub in165(ld, do, clk) | tmp165

  outa[ld]~~                                                    ' default to Shift mode 
  dira[ld]~~
  outa[clk]~                                                    ' Clock is 0-1-0   
  dira[clk]~~
  dira[do]~                                                     ' DO pin is input
                                                                
  outa[ld]~                                                     ' blip Shift/Load line
  outa[ld]~~

  tmp165~                                                       ' clear workspace
  repeat 8                                                      ' read eight bits
    tmp165 := (tmp165 << 1) | ina[do]                           ' get new bit
    outa[clk]~~                                                 ' blip clock
    outa[clk]~  

  return tmp165


pub in165x(ld, do, clk, bits, mode) | tmp165

  outa[ld]~~                                                    ' default to Shift mode   
  dira[ld]~~                                                                              
  outa[clk]~                                                    ' Clock is 0-1-0          
  dira[clk]~~                                                                             
  dira[do]~                                                     ' DO pin is input         
                                                                                          
  outa[ld]~                                                     ' blip Shift/Load line    
  outa[ld]~~                                                                              
                                                                                          
  tmp165~                                                       ' clear workspace
  bits := 1 #> bits <# 32                                       ' limit to legal value         
  repeat bits                                                   ' read n bits         
    tmp165 := (tmp165 << 1) | ina[do]                           ' get new bit             
    outa[clk]~~                                                 ' blip clock              
    outa[clk]~  

  if (mode == 0)                                                ' LSBFIRST result?
    tmp165 ><= bits                                             ' reverse bits
   
  return tmp165 


pub pause(ms) | t

  t := cnt
  repeat ms
    waitcnt(t += MS_001)

 